#!/usr/bin/perl

# Copyright 2011 Ryan Babchishin
my $version = '2.05b';

=begin GPL
    This file is part of Moscrack.

    Moscrack is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, version 3 of the License.

    Moscrack is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moscrack.  If not, see <http://www.gnu.org/licenses/>.
=end GPL
=cut

my $refresh = 2; # Refresh time in seconds


use POSIX qw(strftime);
use Storable qw(lock_retrieve);
use strict;
use DateTime;
use Math::Round;
use Term::ANSIColor;
use DateTime::Format::Duration;
use File::Basename;
use Term::ReadKey;
use CGI;

my $cgi = CGI->new();
print $cgi->header();
print "<font face=arial>\n";

my @nodeState = ('OFFLINE','ONLINE');

my $statusFile = '/haze/status.dat';
my $status;
my $activeNodes;
my $nodeStatus;
my $nodes;
my $key;
my $timer->{rate} = 0;
my $timerInterval = 600;

while() {

	print $cgi->start_form();
	print "Auto-Refresh: ";
	print $cgi->submit(-name=>'refresh', -value=>'Off');
	print $cgi->submit(-name=>'refresh', -value=>'On');

	my ($cols,$rows) = GetTerminalSize();
	my $usedRows = 5; # header uses 5 rows


	$status = lock_retrieve($statusFile);
	$activeNodes = $status->{activeNodes};
	$nodeStatus = $status->{nodeStatus};
	$nodes = $status->{nodes};

	my $percentDone;
	my $complete;
	unless($status->{wordListSize} <= 0) {
		$percentDone = round(($status->{completeWords} / $status->{wordListSize}) * 100).'%';
		$complete = $status->{completeWords}.'/'.$status->{wordListSize};
	} else {
		$complete = $status->{completeWords};
		$percentDone = '';
	}

	if($status->{key}) {
		$key = "<font color=red><b>Key Found<br><font color=blue>".$status->{key};
	}

	my $pid;
	if(-d "/proc/$status->{pid}") {
		$pid = $status->{pid};
	} else {
		$pid = '<font color=red><b> Not running';
	}


	my $b = ' bgcolor=black ';
	my $f = '<b><font color=white>';

	print "<table border=0 cellpadding=8 cellspacing=0>\n";
	print "<tr><td $b colspan=7 align=center><font color=white><b>Moscrack Status Monitor</b></td>\n";
	print "<tr><td $b>".$f."PID</td><td>$pid</td><td $b>".$f."ESSID</td><td>$status->{essid}</td>
		<td rowspan=5 colspan=2 align=center>$key</td></tr>\n";
	print "<tr><td $b>".$f."Word List</td><td>".basename($status->{wordlist})."</td><td $b>".$f."Complete</td><td>$complete</td></tr>\n";
	print "<tr><td $b>".$f."Processes</td><td>$status->{procCount}</td><td $b>".$f."Progress</td><td>$percentDone</td></tr>\n";
	print "<tr><td $b>".$f."Average Rate</td><td>$status->{wordPerSec}</td><td $b>".$f."10 min Sample</td><td>".currentRate()."</td></tr>\n";
	print "</table>";

	my $w = "<font color=white>";
	print $cgi->start_table({-border=>0,-cellspacing=>0,-cellpadding=>8});
	print $cgi->Tr(
                  [
                     $cgi->th({-align=>'left',-bgcolor=>'black'},
			[$w.'Node', $w.'Mult',$w.'LastRes',$w.'Status',$w.'Fail',$w.'Error',$w.'Throt',
				$w.'PID',$w.'Time',$w.'Chunk'])
                  ]
                  );

	foreach my $node (keys %{$nodes}) {
		$usedRows++;
		last if(($rows - $usedRows) <= 0); # Reached terminal limit 

		my ($pid, $chunk, $startTime) = doingWhat($node);

		my $pidSeconds = DateTime->now->epoch - $startTime if $startTime > 0;
		my $pidTime = strftime("\%H:\%M:\%S", gmtime($pidSeconds));

		my $throttleTime;
		if (defined($nodeStatus->{$node}->{throttle})) {
			$throttleTime = $nodeStatus->{$node}->{throttle} - DateTime->now->epoch;
			if ($throttleTime <= 0) {
				$throttleTime = undef; 
			} else {
				$throttleTime = strftime("%M:\%S", gmtime($throttleTime));	
			}
		}


		my $nodeName = $node;
		$nodeName =~ s/^(.{16}).+/$1/;	

		print $cgi->Tr(
                  [
                     $cgi->th({-align=>'left',-bgcolor=>'lightblue'},
			[
				$nodeName,
				$nodes->{$node}->{multiplier},
				$nodeStatus->{$node}->{lastResult},
				$nodeState[$nodeStatus->{$node}->{status}],
				$nodeStatus->{$node}->{fail},
				$nodeStatus->{$node}->{error},
				$throttleTime,
				$pid,
				$pidTime,
				$chunk
			])
		]);
	}


	print $cgi->end_table();
	print "<meta http-equiv=Refresh content=3>" unless $cgi->param('refresh') eq 'Off';
	exit;
}

sub doingWhat {

	my $node = shift;

	foreach my $pid (keys %{$activeNodes}) {
		if ($node eq $activeNodes->{$pid}->{node}) {
			return $pid, $activeNodes->{$pid}->{chunk}, $activeNodes->{$pid}->{startTime};
		}			
	}

}

sub currentRate {
	my $now = DateTime->now->epoch;
	my $currentRate = 0;

	# Timer ended, calculate rate
	if($timer->{next} <= $now) {
		my $processed = $status->{completeWords} - $timer->{words}; 
		my $duration = $now - $timer->{start};
		$currentRate = round($processed / $duration) if $duration > 0;
		$timer->{rate} = $currentRate unless $currentRate == 0;
	}

	# See if timer needs to be reset
	if($timer->{next} <= $now || (! defined($timer->{next}))) {
		# Reset
		$timer->{start} = $now;
		$timer->{next} = $now + $timerInterval;
		$timer->{words} = $status->{completeWords};
	}

	return $timer->{rate};
}

